from enum import Enum
from typing import Iterable

from ward.testing import TestOutcome, TestResult


class ExitCode(Enum):
    SUCCESS = 0
    FAILED = 1
    ERROR = 2
    NO_TESTS_FOUND = 3


def get_exit_code(results: Iterable[TestResult]) -> ExitCode:
    if not results:
        return ExitCode.NO_TESTS_FOUND

    if any(
        r.outcome == TestOutcome.FAIL or r.outcome == TestOutcome.XPASS for r in results
    ):
        exit_code = ExitCode.FAILED
    else:
        exit_code = ExitCode.SUCCESS
    return exit_code


def truncate(s: str, num_chars: int) -> str:
    suffix = "..." if len(s) > num_chars else ""
    return s[: num_chars - len(suffix)] + suffix


def outcome_to_colour(outcome: TestOutcome) -> str:
    return {
        TestOutcome.PASS: "green",
        TestOutcome.SKIP: "blue",
        TestOutcome.FAIL: "red",
        TestOutcome.XFAIL: "magenta",
        TestOutcome.XPASS: "yellow",
    }[outcome]



from timeit import default_timer

import click
import sys
from colorama import init
from ward.collect import (
    get_info_for_modules,
    get_tests_in_modules,
    load_modules,
    search_generally,
)
from ward.suite import Suite
from ward.terminal import SimpleTestResultWrite
from ward.util import get_exit_code

init()

sys.path.append(".")


@click.command()
@click.option(
    "-p", "--path", default=".", type=click.Path(exists=True), help="Path to tests."
)
@click.option(
    "--search",
    help="Search test names, descriptions and module names for the search query and only run matching tests.",
)
@click.option(
    "--fail-limit",
    type=int,
    help="The maximum number of failures that are allowed to occur in a run before it is automatically cancelled.",
)
@click.option(
    "--test-output-style",
    type=click.Choice(
        ["test-per-line", "dots-global", "dots-module"], case_sensitive=False
    ),
)
def run(path, search, fail_limit, test_output_style):
    start_run = default_timer()

    mod_infos = get_info_for_modules(path)
    modules = list(load_modules(mod_infos))
    unfiltered_tests = get_tests_in_modules(modules)
    tests = search_generally(unfiltered_tests, query=search)
    time_to_collect = default_timer() - start_run

    suite = Suite(tests=list(tests))
    test_results = suite.generate_test_runs()

    writer = SimpleTestResultWrite(suite=suite, test_output_style=test_output_style)
    results = writer.output_all_test_results(
        test_results, time_to_collect=time_to_collect, fail_limit=fail_limit
    )
    time_taken = default_timer() - start_run
    writer.output_test_result_summary(results, time_taken)

    exit_code = get_exit_code(results)

    sys.exit(exit_code.value)



import importlib
import importlib.util
import inspect
import os
import pkgutil
from importlib._bootstrap import ModuleSpec
from importlib._bootstrap_external import FileFinder
from typing import Any, Callable, Generator, Iterable, List

from ward.models import WardMeta
from ward.testing import Test, anonymous_tests


def is_test_module(module: pkgutil.ModuleInfo) -> bool:
    return module.name.startswith("test_")


def get_info_for_modules(path: str) -> Generator[pkgutil.ModuleInfo, None, None]:
    # Check for modules at the root of the specified path
    for module in pkgutil.iter_modules([path]):
        yield module

    # Now check for modules in every subdirectory
    for root, dirs, _ in os.walk(path):
        for dir_name in dirs:
            dir_path = os.path.join(root, dir_name)
            for module in pkgutil.iter_modules([dir_path]):
                if is_test_module(module):
                    yield module


def load_modules(modules: Iterable[pkgutil.ModuleInfo]) -> Generator[Any, None, None]:
    for m in modules:
        file_finder: FileFinder = m.module_finder
        spec: ModuleSpec = file_finder.find_spec(m.name)
        mod = importlib.util.module_from_spec(spec)
        if mod.__name__.startswith("test_"):
            spec.loader.exec_module(mod)
            yield mod


def get_tests_in_modules(modules: Iterable) -> Generator[Test, None, None]:
    for mod in modules:
        mod_name = mod.__name__
        # Collect anonymous tests from the module
        anon_tests: List[Callable] = anonymous_tests[mod_name]
        if anon_tests:
            for test_fn in anon_tests:
                meta: WardMeta = getattr(test_fn, "ward_meta")
                yield Test(
                    fn=test_fn,
                    module_name=mod_name,
                    marker=meta.marker,
                    description=meta.description or "",
                )


def search_generally(
    tests: Iterable[Test], query: str = ""
) -> Generator[Test, None, None]:
    if not query:
        yield from tests

    for test in tests:
        description = test.description or ""
        if (
            query in description
            or query in f"{test.module_name}."
            or query in inspect.getsource(test.fn)
            or query in test.qualified_name
        ):
            yield test



import inspect
import math
from dataclasses import dataclass
from typing import Type, Any, List, Callable, Dict, Tuple, Optional, Iterable
from unittest.mock import _Call


class raises:
    def __init__(self, expected_ex_type: Type[Exception]):
        self.expected_ex_type = expected_ex_type

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type is not self.expected_ex_type:
            raise AssertionError(
                f"Expected exception {self.expected_ex_type}, but {exc_type} was raised instead."
            )
        return True


@dataclass
class Expected:
    this: Any
    op: str
    that: Optional[Any]
    op_args: Tuple
    op_kwargs: Dict
    success: bool = True


class ExpectationFailed(Exception):
    def __init__(self, message: str, history: List[Expected]):
        self.message = message
        self.history = history


class expect:
    def __init__(self, this: Any):
        self.this = this
        self.history: List[Expected] = []

    def equals(self, expected: Any):
        return self._handle_expect(self.this == expected, that=expected)

    def not_equals(self, that: Any):
        return self._handle_expect(self.this != that, that=that)

    def less_than(self, that: Any):
        return self._handle_expect(self.this < that, that=that)

    def not_less_than(self, that: Any):
        return self._handle_expect(not self.this < that, that=that)

    def less_than_or_equals(self, that: Any):
        return self._handle_expect(self.this <= that, that=that)

    def not_less_than_or_equals(self, that: Any):
        return self._handle_expect(not self.this <= that, that=that)

    def greater_than(self, that: Any):
        return self._handle_expect(self.this > that, that=that)

    def not_greater_than(self, that: Any):
        return self._handle_expect(not self.this > that, that=that)

    def greater_than_or_equals(self, that: Any):
        return self._handle_expect(self.this >= that, that=that)

    def not_greater_than_or_equals(self, that: Any):
        return self._handle_expect(not self.this >= that, that=that)

    def contains(self, that: Any):
        return self._handle_expect(that in self.this, that=that)

    def not_contains(self, that: Any):
        return self._handle_expect(that not in self.this, that=that)

    def contained_in(self, that: Iterable[Any]):
        return self._handle_expect(self.this in that, that=that)

    def not_contained_in(self, that: Iterable[Any]):
        return self._handle_expect(self.this not in that, that=that)

    def has_length(self, length: int):
        return self._handle_expect(len(self.this) == length, that=length)

    def not_has_length(self, length: int):
        return self._handle_expect(len(self.this) != length, that=length)

    def instance_of(self, type: Type):
        return self._handle_expect(isinstance(self.this, type), that=type)

    def not_instance_of(self, type: Type):
        return self._handle_expect(not isinstance(self.this, type), that=type)

    def satisfies(self, predicate: Callable[["expect"], bool]):
        return self._handle_expect(predicate(self.this), that=predicate)

    def not_satisfies(self, predicate: Callable[["expect"], bool]):
        return self._handle_expect(not predicate(self.this), that=predicate)

    def identical_to(self, that: Any):
        return self._handle_expect(self.this is that, that=that)

    def not_identical_to(self, that: Any):
        return self._handle_expect(self.this is not that, that=that)

    def approx(self, that: Any, rel_tol: float = 1e-9, abs_tol: float = 0.0):
        return self._handle_expect(
            math.isclose(self.this, that, abs_tol=abs_tol, rel_tol=rel_tol),
            that=that,
            rel_tol=rel_tol,
            abs_tol=abs_tol,
        )

    def not_approx(self, that: Any, rel_tol: float = 1e-9, abs_tol: float = 0.0):
        return self._handle_expect(
            not math.isclose(self.this, that, abs_tol=abs_tol, rel_tol=rel_tol),
            that=that,
            rel_tol=rel_tol,
            abs_tol=abs_tol,
        )

    def called(self):
        return self._handle_expect(self.this.called)

    def not_called(self):
        return self._handle_expect(not self.this.called)

    def called_once_with(self, *args, **kwargs):
        try:
            self.this.assert_called_once_with(*args, **kwargs)
            passed = True
        except AssertionError:
            passed = False
        return self._handle_expect(passed, *args, **kwargs)

    def called_with(self, *args, **kwargs):
        try:
            self.this.assert_called_with(*args, **kwargs)
            passed = True
        except AssertionError:
            passed = False
        return self._handle_expect(passed, *args, **kwargs)

    def has_calls(self, calls: List[_Call], any_order: bool = False):
        try:
            self.this.assert_has_calls(calls, any_order=any_order)
            passed = True
        except AssertionError:
            passed = False
        return self._handle_expect(passed, calls=calls, any_order=any_order)

    def _store_in_history(
        self,
        result: bool,
        called_with_args: Tuple[Any],
        called_with_kwargs: Dict[str, Any],
        that: Any = None,
    ) -> bool:
        self.history.append(
            Expected(
                this=self.this,
                op=inspect.stack()[2].function,  # :)
                that=that,
                success=result,
                op_args=called_with_args,
                op_kwargs=called_with_kwargs,
            )
        )
        return result

    def _fail_if_false(self, val: bool) -> bool:
        if val:
            return True
        raise ExpectationFailed("expectation failed", self.history)

    def _handle_expect(
        self, result: bool, *args, that: Any = None, **kwargs
    ) -> "expect":
        self._store_in_history(
            result, that=that, called_with_args=args, called_with_kwargs=kwargs
        )
        self._fail_if_false(result)
        return self



import platform

from setuptools import setup

version = "0.20.0a0"
description = "A modern Python 3 test framework for finding and fixing flaws faster."
with open("README.md", "r") as fh:
    if platform.system() != "Windows":
        long_description = fh.read()
    else:
        long_description = description

setup(
    name="ward",
    version=version,
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/darrenburns/ward",
    author="Darren Burns",
    author_email="darrenb900@gmail.com",
    license="MIT",
    packages=["ward"],
    python_requires=">=3.6",
    entry_points={"console_scripts": ["ward=ward.run:run"]},
    install_requires=[
        "colorama==0.4.1",
        "termcolor==1.1.0",
        "dataclasses==0.6",
        "click==7.0",
    ],
)
